import 'dart:convert';
import 'dart:io';

import 'package:dcoupon_sdk_flutter_module/constants.dart';
import 'package:dcoupon_sdk_flutter_module/httprequest.dart';
import 'package:dcoupon_sdk_flutter_module/translations.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:flutter_cupertino_date_picker/flutter_cupertino_date_picker.dart';
import 'package:flutter_html/flutter_html.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:html_unescape/html_unescape.dart';
import 'package:intl/intl.dart';

///  @author: jvrodrigo
class EnrollmentFormView extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
        home: EnrollmentForm(),
        localizationsDelegates: [
          const TranslationsDelegate(),
          GlobalMaterialLocalizations.delegate,
          GlobalWidgetsLocalizations.delegate,
          GlobalCupertinoLocalizations.delegate
        ],
        supportedLocales: [
          const Locale('en'),
          const Locale('es'),
        ],
        debugShowCheckedModeBanner: false);
  }
}

class EnrollmentForm extends StatefulWidget {
  EnrollmentForm() : super();

  @override
  _EnrollmentForm createState() => _EnrollmentForm();
}

class _EnrollmentForm extends State<EnrollmentForm> {
  static const MethodChannel _channel =
      const MethodChannel(Constants.CHANNEL_NAME_VIEW);
  var legalTextPP, legalTextTOU;
  var email, alias, gender, birthdate;
  var _checkboxIsSelected = false;
  var _isVisibleTextLegalInfo = true;
  var _isEnableNextBtn = false;
  var _formKey = GlobalKey<FormState>();
  String _currentGender;
  DateTime selectedDate;
  var unescape = new HtmlUnescape();
  List<dynamic> legalDocumentsToAccept = new List<dynamic>();
  HtmlEscape htmlEscape = HtmlEscape();
  DateFormat dateFormat;
  DateFormat dateFormatToSend = new DateFormat("yyyy-MM-dd");
  DateTime _maxDateAllowed = new DateTime(
      DateTime.now().year - 16, DateTime.now().month, DateTime.now().day);
  ScrollController _scrollController;

  @override
  void initState() {
    super.initState();
    dateFormat = new DateFormat.yMd(Locale.cachedLocaleString);
    _channel.setMethodCallHandler(_receiveFromHost);
    _scrollController = ScrollController();
    try {
      ready();
      selectedDate = _maxDateAllowed;
    } catch (ex) {
      print(ex.toString());
    }
  }

  void ready() async {
    await _channel.invokeMethod("ready", "true");
  }

  @override
  void setState(fn) {
    super.setState(fn);
  }

  Future<void> _receiveFromHost(MethodCall call) async {
    //print("------------------------ _receiveFromHost -----------------");
    setState(() {
      try {
        var jsonResponse =
            json.decode(call.arguments[HttpInterface.JSON_RESPONSE].toString());
        var jsonForm = jsonResponse[Constants.FORM];
        var jsonLegalDocs =
            new List<dynamic>.from(jsonForm[Constants.LEGAL_DOCUMENTS]);
        jsonLegalDocs.asMap().forEach((key, value) {
          if (value.containsKey(Constants.COUNTRY)) {
            String country = value[Constants.COUNTRY];
            String docType = value[Constants.DOC_TYPE];
            String docUuid = value[Constants.DOC_UUID];
            var content = value[Constants.CONTENT];
            content = content[country];
            var legalDoc = content[Constants.LEGAL_DOC];
            switch (docType) {
              case Constants.PP:
                legalTextPP = unescape.convert(legalDoc);
                break;
              case Constants.TOU:
                legalTextTOU = unescape.convert(legalDoc);
                break;
              default:
                break;
            }
            Map<dynamic, dynamic> object = new Map<dynamic, dynamic>();
            object.putIfAbsent(Constants.DOC_UUID, () => docUuid);
            object.putIfAbsent(Constants.DOC_TYPE, () => docType);
            legalDocumentsToAccept.add(object);
          }
        });
        //user-data
        var user = jsonForm[Constants.USER];
        email = user[Constants.EMAIL];
        alias = user[Constants.ALIAS];
        gender = _validateGender(user[Constants.GENDER])
            ? user[Constants.GENDER]
            : Constants.RATHER_NOT_SAY;
        birthdate = _validateBirthdate(user[Constants.BIRTHDATE])
            ? user[Constants.BIRTHDATE]
            : dateFormatToSend.format(_maxDateAllowed);
        _currentGender = gender;
        selectedDate = DateTime.parse(birthdate);
      } catch (ex) {
        print(ex.toString());
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        body: DefaultTabController(
      length: 3,
      child: Scaffold(
        appBar: AppBar(
            iconTheme: IconThemeData(color: Colors.black),
            brightness: Brightness.dark,
            backgroundColor: Colors.white,
            bottom: TabBar(
              labelColor: Colors.black,
              indicatorColor: Colors.orange,
              tabs: [
                Tab(text: Translations.of(context).text('dcoupon_welcome')),
                Tab(text: Translations.of(context).text('dcoupon_legal_pp')),
                Tab(text: Translations.of(context).text('dcoupon_legal_tou')),
              ],
            ),
            title: new Image(
              fit: BoxFit.scaleDown,
              image: new AssetImage('lib/assets/images/dcoupon_logo.png'),
            )),
        body: TabBarView(
          children: [
            Container(
              child: SingleChildScrollView(
                controller: _scrollController,
                padding: EdgeInsets.all(8.0),
                child: Visibility(
                  visible: _isVisibleTextLegalInfo,
                  child: new Column(
                    children: <Widget>[
                      new Row(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: [
                          Checkbox(
                              value: _checkboxIsSelected,
                              onChanged: (bool value) {
                                setState(() {
                                  _checkboxIsSelected = value;
                                  _isEnableNextBtn =
                                      _checkboxIsSelected ? true : false;
                                  if (_isEnableNextBtn) {
                                    _scrollBottom();
                                  }
                                });
                              }),
                          Text(Translations.of(context)
                              .text('dcoupon_check_box_text')),
                        ],
                      ),
                      Container(
                          height: MediaQuery.of(context).size.height * 0.6,
                          child: SingleChildScrollView(
                              child: Html(
                            data: Translations.of(context)
                                .text('dcoupon_legal_info_text'),
                            //Optional parameters:
                            linkStyle: const TextStyle(
                              color: Colors.blue,
                              decorationColor: Colors.blueAccent,
                              decoration: TextDecoration.underline,
                            ),
                          ))),
                      new Column(
                          mainAxisSize: MainAxisSize.max,
                          mainAxisAlignment: MainAxisAlignment.end,
                          children: <Widget>[
                            Align(
                                alignment: Alignment.bottomRight,
                                child: RaisedButton(
                                    color: Colors.blueAccent,
                                    onPressed: (_isEnableNextBtn)
                                        ? () => isEnableNextBtn()
                                        : null,
                                    child: new Container(
                                        child: Row(
                                      crossAxisAlignment:
                                          CrossAxisAlignment.center,
                                      mainAxisAlignment:
                                          MainAxisAlignment.spaceBetween,
                                      mainAxisSize: MainAxisSize.min,
                                      children: <Widget>[
                                        Text(
                                            Translations.of(context)
                                                .text('dcoupon_next'),
                                            style: TextStyle(
                                                fontSize: 16,
                                                color: Colors.white)),
                                        Icon(
                                          Icons.arrow_forward,
                                          color: Colors.white,
                                        )
                                      ],
                                    ))))
                          ]),
                    ],
                  ),
                  replacement: Column(
                    children: <Widget>[
                      Form(
                          key: _formKey,
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: <Widget>[
                              TextFormField(
                                readOnly: true,
                                decoration: InputDecoration(
                                    labelText: Translations.of(context)
                                        .text('dcoupon_email')),
                                initialValue: email,
                                validator: (value) {
                                  if (value.isEmpty) {
                                    return (Translations.of(context)
                                            .text('dcoupon_missed_parameter') +
                                        " " +
                                        Translations.of(context)
                                            .text('dcoupon_email'));
                                  }
                                  return null;
                                },
                              ),
                              TextFormField(
                                decoration: InputDecoration(
                                    labelText: Translations.of(context)
                                        .text('dcoupon_alias')),
                                initialValue: alias,
                                onChanged: (value) {
                                  if (value.length > 0)
                                    setState(() {
                                      alias = value;
                                    });
                                },
                                validator: (value) {
                                  if (value.isEmpty) {
                                    return (Translations.of(context)
                                            .text('dcoupon_missed_parameter') +
                                        " " +
                                        Translations.of(context)
                                            .text('dcoupon_alias'));
                                  }
                                  return null;
                                },
                              ),
                              new Row(children: <Widget>[
                                Expanded(
                                    child: Container(
                                  margin: const EdgeInsets.only(top: 4),
                                  height: 70,
                                  child: Column(
                                    verticalDirection: VerticalDirection.down,
                                    mainAxisAlignment: MainAxisAlignment.end,
                                    crossAxisAlignment:
                                        CrossAxisAlignment.start,
                                    children: <Widget>[
                                      new Text(
                                        Translations.of(context)
                                            .text('dcoupon_gender'),
                                        style: TextStyle(
                                            fontSize: 12,
                                            color: Colors.grey[700]),
                                      ),
                                      new DropdownButton(
                                        isExpanded: true,
                                        hint: new Text(Translations.of(context)
                                                .text(
                                                    'dcoupon_missed_parameter') +
                                            " " +
                                            Translations.of(context)
                                                .text('dcoupon_gender')),
                                        value: _currentGender,
                                        items: [
                                          DropdownMenuItem(
                                            value: Constants.MALE,
                                            child: Text(
                                              Translations.of(context)
                                                  .text('dcoupon_gender_male'),
                                            ),
                                          ),
                                          DropdownMenuItem(
                                            value: Constants.FEMALE,
                                            child: Text(
                                              Translations.of(context).text(
                                                  'dcoupon_gender_female'),
                                            ),
                                          ),
                                          DropdownMenuItem(
                                            value: Constants.OTHER,
                                            child: Text(
                                              Translations.of(context)
                                                  .text('dcoupon_gender_other'),
                                            ),
                                          ),
                                          DropdownMenuItem(
                                            value: Constants.RATHER_NOT_SAY,
                                            child: Text(
                                              Translations.of(context).text(
                                                  'dcoupon_gender_rather_not_say'),
                                            ),
                                          ),
                                        ],
                                        onChanged: changedDropDownItemGender,
                                      )
                                    ],
                                  ),
                                ))
                              ]),
                              new Row(children: <Widget>[
                                Expanded(
                                    child: Container(
                                        margin: const EdgeInsets.only(top: 2),
                                        height: 70,
                                        child: Column(
                                            verticalDirection:
                                                VerticalDirection.down,
                                            mainAxisAlignment:
                                                MainAxisAlignment.end,
                                            crossAxisAlignment:
                                                CrossAxisAlignment.start,
                                            children: <Widget>[
                                              Text(
                                                  Translations.of(context).text(
                                                      'dcoupon_birthdate'),
                                                  style: TextStyle(
                                                      fontSize: 12,
                                                      color: Colors.grey[700])),
                                              new SizedBox(
                                                  width: double.infinity,
                                                  // height: double.infinity,
                                                  child: RaisedButton(
                                                      color: Colors.white,
                                                      onPressed: () =>
                                                          _selectDate(context),
                                                      child: Column(
                                                        mainAxisAlignment:
                                                            MainAxisAlignment
                                                                .start,
                                                        children: <Widget>[
                                                          Text(
                                                            dateFormat.format(
                                                                selectedDate),
                                                            style: TextStyle(
                                                                fontSize: 16,
                                                                color: Colors
                                                                    .black),
                                                          ),
                                                        ],
                                                      )))
                                            ])))
                              ]),
                              new Row(
                                  mainAxisSize: MainAxisSize.max,
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  crossAxisAlignment: CrossAxisAlignment.center,
                                  children: <Widget>[
                                    RaisedButton(
                                        color: Colors.grey,
                                        onPressed: () {
                                          setState(() {
                                            _isVisibleTextLegalInfo = true;
                                          });
                                          // Validate will return true if the form is valid, or false if
                                          // the form is invalid.
                                          if (_formKey.currentState
                                              .validate()) {
                                            // Process data.
                                          }
                                        },
                                        child: Row(
                                          crossAxisAlignment:
                                              CrossAxisAlignment.center,
                                          mainAxisAlignment:
                                              MainAxisAlignment.start,
                                          mainAxisSize: MainAxisSize.min,
                                          children: <Widget>[
                                            Icon(Icons.arrow_back,
                                                color: Colors.white),
                                            Text(
                                                Translations.of(context)
                                                    .text('dcoupon_back'),
                                                style: TextStyle(
                                                    fontSize: 16,
                                                    color: Colors.white)),
                                          ],
                                        )),
                                    RaisedButton(
                                        color: Colors.blueAccent,
                                        onPressed: () {
                                          setState(() {});
                                          // Validate will return true if the form is valid, or false if
                                          // the form is invalid.
                                          if (_formKey.currentState
                                              .validate()) {
                                            // Process data.
                                            acceptEnrollment();
                                          }
                                        },
                                        child: Row(
                                          crossAxisAlignment:
                                              CrossAxisAlignment.center,
                                          mainAxisAlignment:
                                              MainAxisAlignment.start,
                                          mainAxisSize: MainAxisSize.min,
                                          children: <Widget>[
                                            Text(
                                                Translations.of(context)
                                                    .text('dcoupon_accept'),
                                                style: TextStyle(
                                                    fontSize: 16,
                                                    color: Colors.white)),
                                            Icon(Icons.check,
                                                color: Colors.white)
                                          ],
                                        ))
                                  ]),
                            ],
                          )),
                    ],
                  ),
                ),
              ),
            ),
            new Container(
              child: SingleChildScrollView(
                padding: EdgeInsets.all(8.0),
                child: Html(
                  data: legalTextPP,
                  linkStyle: const TextStyle(
                    color: Colors.blue,
                    decorationColor: Colors.blueAccent,
                    decoration: TextDecoration.underline,
                  ),
                ),
              ),
            ),
            new Container(
              child: SingleChildScrollView(
                padding: EdgeInsets.all(8.0),
                child: Html(
                  data: legalTextTOU,
                  linkStyle: const TextStyle(
                    color: Colors.blue,
                    decorationColor: Colors.blueAccent,
                    decoration: TextDecoration.underline,
                  ),
                ),
              ),
            ),
          ],
        ),
      ),
    ));
  }

  void isEnableNextBtn() {
    setState(() {
      _isVisibleTextLegalInfo = false;
    });
  }

  void acceptEnrollment() {
    _channel.invokeMethod(Constants.THIRD_PARTY_ACCEPT_ENROLLMENT,
        jsonEncode(User(email, alias, gender, birthdate, legalDocumentsToAccept)));
  }

  Future<Null> _selectDate(BuildContext context) async {
    if (Platform.isIOS) {
      DatePicker.showDatePicker(context,
          initialDateTime: selectedDate,
          minDateTime: DateTime(1900),
          maxDateTime: _maxDateAllowed, onConfirm: (picked, list) => _updateSelectedDate(picked));
    } else {
      final DateTime picked = await showDatePicker(
          context: context,
          initialDate: selectedDate,
          firstDate: DateTime(1900),
          lastDate: _maxDateAllowed);
      _updateSelectedDate(picked);
    }
  }

  void _updateSelectedDate(DateTime picked) {
    if (picked != null &&
        picked != selectedDate &&
        picked.millisecondsSinceEpoch <
            _maxDateAllowed.millisecondsSinceEpoch) {
      setState(() {
        selectedDate = picked;
        birthdate = dateFormatToSend.format(selectedDate);
      });
    }
  }

  void changedDropDownItemGender(String selectedGender) {
    setState(() {
      _currentGender = selectedGender;
      gender = _currentGender;
    });
  }

  bool _validateGender(String gender) {
    switch (gender) {
      case Constants.MALE:
      case Constants.FEMALE:
      case Constants.OTHER:
      case Constants.RATHER_NOT_SAY:
        return true;
        break;
      default:
        return false;
        break;
    }
  }

  bool _validateBirthdate(String date) {
    try {
      return _maxDateAllowed.isAfter(DateTime.parse(date)) ? true : false;
    } catch (ex) {
      print(ex.toString());
      return false;
    }
  }

  void _scrollBottom() {
    _scrollController.animateTo(_scrollController.position.maxScrollExtent,
        duration: Duration(milliseconds: 200), curve: Curves.linear);
  }
}

class User {
  final String email, alias, gender, birthdate;
  final List legalDocumentsAccepted;

  User.fromMappedJson(Map<dynamic, dynamic> json)
      : email = json[Constants.EMAIL],
        alias = json[Constants.ALIAS],
        gender = json[Constants.GENDER],
        birthdate = json[Constants.BIRTHDATE],
        legalDocumentsAccepted = json[Constants.LEGAL_DOCUMENTS_ACCEPTED];

  Map<dynamic, dynamic> toJson() => {
        Constants.EMAIL : email,
        Constants.ALIAS : alias,
        Constants.GENDER : gender,
        Constants.BIRTHDATE : birthdate,
        Constants.LEGAL_DOCUMENTS_ACCEPTED : legalDocumentsAccepted
      };

  User(this.email, this.alias, this.gender, this.birthdate,
      this.legalDocumentsAccepted);
}
