/// @author: jvrodrigo

import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'constants.dart';
import 'httprequest.dart';

class HttpService extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(home: DcouponHttpChannel());
  }
}

class DcouponHttpChannel extends StatefulWidget {
  DcouponHttpChannel() : super();

  @override
  _HttpService createState() => _HttpService();
}

class _HttpService extends State<DcouponHttpChannel> {
  static const MethodChannel _channel = const MethodChannel(Constants.CHANNEL_NAME_HTTP);

  @override
  initState() {
    super.initState();
    //print("DcouponHttpChannel._HttpService.initState -> " + _channel.name);
    _channel.setMethodCallHandler(_receiveFromHost);
  }

  Future<dynamic> invokeMethod(String method, dynamic args) async {
    return await _channel.invokeMethod(method, args);
  }

  Future<void> _receiveFromHost(MethodCall call) async {
    // stream controller to handle the http response
    var streamController = StreamController<HttpResponse>();
    streamController.stream.listen((data) {
      try {
        _channel.invokeMethod(call.method, data.toJson());
      } catch (e) {
        print(e.toString());
      }
    });
    try {
      //print("DcouponHttpChannel._HttpService.method -> " + call.method);
      HttpRequest httpRequest = new HttpRequest();
      httpRequest.setMethod(call.method);
      httpRequest.setArguments(call.arguments);
      httpRequest.setDomain(call.arguments[Constants.CLIENT_DOMAIN]);
      if (call.arguments.containsKey(Constants.PATH_PARAMETER))
        httpRequest.setPathParameter(call.arguments[Constants.PATH_PARAMETER]);
      httpRequest.setStreamController(streamController);
      httpRequest.execute();
    } on Exception catch (ex) {
      print(ex.toString());
      HttpResponse httpResponse = new HttpResponse();
      httpResponse.statusCode = HttpInterface.HTTP_INTERNAL_ERROR;
      httpResponse.headers = new Map();
      httpResponse.body = ex.toString();
      streamController.sink.add(httpResponse);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        body: Center(
            child: Column(mainAxisAlignment: MainAxisAlignment.center, children: <Widget>[
      new Image(image: new AssetImage('lib/assets/images/dcoupon_logo.png')),
      Text("Opps!, it is a black hole ;P")
    ])));
  }

  @override
  dispose() {
    super.dispose();
  }
}
