/// @author: jvrodrigo

import 'package:dcoupon_sdk_flutter_module/constants.dart';
import 'package:dcoupon_sdk_flutter_module/httprequest.dart';

class Services {
  Map<dynamic, dynamic> schema;
  List properties;
  List payload;
  String domain;
  dynamic pathParameter;

  Services() {
    schema = new Map();
    properties = new List();
    payload = new List();
  }

  void setDomain(String domain) {
    this.domain = domain;
  }

  void setPathParameter(dynamic pathParameter) {
    this.pathParameter = pathParameter;
  }

  Map<dynamic, dynamic> getSchema(String serviceName) {
    switch (serviceName) {
      case Constants.THIRD_PARTY_LOGIN:
        properties.add(Constants.CONTENT_TYPE);
        properties.add(Constants.CLIENT_API_TOKEN);
        properties.add(Constants.CLIENT_SECRET_KEY);
        payload.add(Constants.CLIENT_COUNTRY);
        payload.add(Constants.EMAIL);
        payload.add(Constants.EXTERNAL_ID);
        payload.add(Constants.ALIAS);
        payload.add(Constants.GENDER);
        payload.add(Constants.BIRTHDATE);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.PAYLOAD, () => payload);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.POST);
        schema.putIfAbsent(HttpInterface.URL,
            () => HttpInterface.URLS_API_THIRD_PARTY_LOGIN[domain] + HttpInterface.PATH_THIRD_PARTY_LOGIN);
        return schema;
        break;
      case Constants.THIRD_PARTY_ENROLLMENT_FORM:
        properties.add(Constants.CONTENT_TYPE);
        properties.add(Constants.CLIENT_API_TOKEN);
        properties.add(Constants.JWT);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.PAYLOAD, () => payload);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.GET);
        schema.putIfAbsent(HttpInterface.URL,
            () => HttpInterface.URLS_API_THIRD_PARTY_LOGIN[domain] + HttpInterface.PATH_THIRD_PARTY_ENROLLMENT_FORM);
        return schema;
        break;
      case Constants.THIRD_PARTY_ACCEPT_ENROLLMENT:
        properties.add(Constants.CONTENT_TYPE);
        properties.add(Constants.CLIENT_API_TOKEN);
        properties.add(Constants.CLIENT_SECRET_KEY);
        payload.add(Constants.JWT);
        payload.add(Constants.EMAIL);
        payload.add(Constants.EXTERNAL_ID);
        payload.add(Constants.ALIAS);
        payload.add(Constants.GENDER);
        payload.add(Constants.BIRTHDATE);
        payload.add(Constants.LEGAL_DOCUMENTS_ACCEPTED);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.PAYLOAD, () => payload);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.POST);
        schema.putIfAbsent(HttpInterface.URL,
            () => HttpInterface.URLS_API_THIRD_PARTY_LOGIN[domain] + HttpInterface.PATH_THIRD_PARTY_ACCEPT_ENROLLMENT);
        return schema;
        break;
      case Constants.USER_COUPONS:
        properties.add(HttpInterface.AUTHORIZATION);
        payload.add(Constants.START);
        payload.add(Constants.LIMIT);
        payload.add(Constants.SORT_BY);
        payload.add(Constants.LANG);
        payload.add(Constants.RETAILER_TOKENS);
        payload.add(Constants.COMPANY_TOKENS);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.PAYLOAD, () => payload);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.GET);
        schema.putIfAbsent(
            HttpInterface.URL, () => HttpInterface.URLS_API_COUPONS[domain] + HttpInterface.PATH_USER_COUPONS);
        return schema;
        break;
      case Constants.USER_COUPON:
        properties.add(HttpInterface.AUTHORIZATION);
        payload.add(Constants.LANG);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.PAYLOAD, () => payload);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.GET);
        schema.putIfAbsent(HttpInterface.URL,
            () => HttpInterface.URLS_API_COUPONS[domain] + HttpInterface.PATH_USER_COUPON + pathParameter);
        return schema;
        break;
      case Constants.USER_COUPON_ACTIVATE:
        properties.add(HttpInterface.AUTHORIZATION);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.GET);
        schema.putIfAbsent(HttpInterface.URL,
            () => HttpInterface.URLS_API_COUPONS[domain] + HttpInterface.PATH_USER_COUPON_ACTIVATE + pathParameter);
        return schema;
        break;
      case Constants.USER_COUPON_DEACTIVATE:
        properties.add(HttpInterface.AUTHORIZATION);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.GET);
        schema.putIfAbsent(HttpInterface.URL,
            () => HttpInterface.URLS_API_COUPONS[domain] + HttpInterface.PATH_USER_COUPON_DEACTIVATE + pathParameter);
        return schema;
        break;
      case Constants.USER_REDEMPTION_CODE:
        properties.add(HttpInterface.AUTHORIZATION);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.GET);
        schema.putIfAbsent(
            HttpInterface.URL, () => HttpInterface.URLS_API_COUPONS[domain] + HttpInterface.PATH_USER_REDEMPTION_CODE);
        return schema;
        break;
      case Constants.USER_CREATE_COUPONS:
        properties.add(HttpInterface.AUTHORIZATION);
        properties.add(Constants.CONTENT_TYPE);
        payload.add(Constants.API_TOKEN_PUBLISHER);
        payload.add(Constants.META_COUPON_TOKENS);
        payload.add(Constants.UCUPON_TYPE);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.PAYLOAD, () => payload);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.POST);
        schema.putIfAbsent(HttpInterface.URL,
            () => HttpInterface.URLS_API_COUPONS[domain] + HttpInterface.PATH_USER_CREATE_MULTIPLE_COUPONS);
        return schema;
        break;
      case Constants.PUBLISHER_PROMOTIONS:
        properties.add(Constants.CLIENT_API_TOKEN);
        properties.add(Constants.CLIENT_SECRET_KEY);
        payload.add(Constants.PUBLISHER_ID);
        payload.add(Constants.LANG);
        payload.add(Constants.COUNTRY);
        payload.add(Constants.START);
        payload.add(Constants.LIMIT);
        payload.add(Constants.CATEGORY_IDS);
        payload.add(Constants.STORE_IDS);
        payload.add(Constants.LONGITUDE);
        payload.add(Constants.LATITUDE);
        payload.add(Constants.RADIUS);
        payload.add(Constants.TEXT_SEARCH);
        payload.add(Constants.ORDER_BY);
        payload.add(Constants.RETAILER_TOKENS);
        payload.add(Constants.COMPANY_TOKENS);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.PAYLOAD, () => payload);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.GET);
        schema.putIfAbsent(HttpInterface.URL,
            () => HttpInterface.URLS_API_PUBLISHER[domain] + HttpInterface.PATH_PUBLISHER_PROMOTIONS);
        return schema;
        break;
      case Constants.PUBLISHER_PROMOTION:
        properties.add(Constants.CLIENT_API_TOKEN);
        properties.add(Constants.CLIENT_SECRET_KEY);
        payload.add(Constants.PUBLISHER_ID);
        payload.add(Constants.LANG);
        payload.add(Constants.COUNTRY);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.PAYLOAD, () => payload);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.GET);
        schema.putIfAbsent(HttpInterface.URL,
            () => HttpInterface.URLS_API_PUBLISHER[domain] + HttpInterface.PATH_PUBLISHER_PROMOTION + pathParameter);
        return schema;
        break;
      case Constants.PUBLISHER_CATEGORIES:
        properties.add(Constants.CLIENT_API_TOKEN);
        properties.add(Constants.CLIENT_SECRET_KEY);
        payload.add(Constants.PUBLISHER_ID);
        payload.add(Constants.LANG);
        payload.add(Constants.COUNTRY);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.PAYLOAD, () => payload);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.GET);
        schema.putIfAbsent(HttpInterface.URL,
            () => HttpInterface.URLS_API_PUBLISHER[domain] + HttpInterface.PATH_PUBLISHER_CATEGORIES);
        return schema;
        break;
      case Constants.PUBLISHER_COMPANIES:
        properties.add(Constants.CLIENT_API_TOKEN);
        properties.add(Constants.CLIENT_SECRET_KEY);
        payload.add(Constants.PUBLISHER_ID);
        payload.add(Constants.LANG);
        payload.add(Constants.COUNTRY);
        payload.add(Constants.START);
        payload.add(Constants.LIMIT);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.PAYLOAD, () => payload);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.GET);
        schema.putIfAbsent(
            HttpInterface.URL, () => HttpInterface.URLS_API_PUBLISHER[domain] + HttpInterface.PATH_PUBLISHER_COMPANIES);
        return schema;
        break;
      case Constants.PUBLISHER_RETAILERS:
        properties.add(Constants.CLIENT_API_TOKEN);
        properties.add(Constants.CLIENT_SECRET_KEY);
        payload.add(Constants.PUBLISHER_ID);
        payload.add(Constants.LANG);
        payload.add(Constants.COUNTRY);
        payload.add(Constants.START);
        payload.add(Constants.LIMIT);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.PAYLOAD, () => payload);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.GET);
        schema.putIfAbsent(
            HttpInterface.URL, () => HttpInterface.URLS_API_PUBLISHER[domain] + HttpInterface.PATH_PUBLISHER_RETAILERS);
        return schema;
        break;
      case Constants.PUBLISHER_STORES:
        properties.add(Constants.CLIENT_API_TOKEN);
        properties.add(Constants.CLIENT_SECRET_KEY);
        payload.add(Constants.PUBLISHER_ID);
        payload.add(Constants.LANG);
        payload.add(Constants.COUNTRY);
        payload.add(Constants.START);
        payload.add(Constants.LIMIT);
        payload.add(Constants.PROMOTION_TOKEN);
        payload.add(Constants.ZIPCODE);
        payload.add(Constants.RADIUS);
        payload.add(Constants.LONGITUDE);
        payload.add(Constants.LATITUDE);
        schema.putIfAbsent(HttpInterface.PROPERTIES, () => properties);
        schema.putIfAbsent(HttpInterface.PAYLOAD, () => payload);
        schema.putIfAbsent(HttpInterface.HTTP_METHOD, () => HttpInterface.GET);
        schema.putIfAbsent(
            HttpInterface.URL, () => HttpInterface.URLS_API_PUBLISHER[domain] + HttpInterface.PATH_PUBLISHER_STORES);
        return schema;
        break;
      default:
        return null;
        break;
    }
  }
}
